#!/bin/sh

function about(){
 echo "#########################################################";
 echo "# Copyright (c) 2023 Alexey Kuryakin kouriakine@mail.ru #";
 echo "# Control Panel for Diesel Pascal interpreter language. #";
 echo "#########################################################";
};

readonly icon_run="/usr/share/pixmaps/CrossMachine.png";
readonly icon_ide="/usr/share/pixmaps/CrossDesigner.png";
readonly initscript="/opt/diesel/examples/tetris.lm9";
readonly envoptconf="$HOME/.DieselPascal/EnvOptions.xml";
readonly syncolconf="$HOME/.DieselPascal/SynColors.cfg";
readonly runhistory="$HOME/.DieselPascal/run.history";
readonly idehistory="$HOME/.DieselPascal/ide.history";
readonly txthistory="$HOME/.DieselPascal/history.txt";
readonly maxhistory="1024";

function launch(){
 echo launch: "$@";
 "$@" >/dev/null 2>&1 & disown;
};

function makeuniq(){
 if [ -n "$1" ] && [ -e "$1" ]; then
  cp $1 $1.tmp;
  cat $1.tmp | uniq | tail -n $maxhistory > $1;
  rm -f $1.tmp;
 fi;
};

function saveuniq(){
 if [ -n "$1" ] && [ -n "$2" ]; then
  echo "$2" >> $1;
  makeuniq $1;
 fi;
};

function backup_file(){
 if [ -n "$1" ] && [ -e "$1" ]; then
  local dir="$(dirname $1)";
  local name="$(basename $1)";
  [ -e $dir/backup ] || mkdir $dir/backup;
  cp -f $1 $dir/backup/$name && { echo "backup: $1 => $dir/backup/$name"; };
 fi;
};

function cleanup_backup_file(){
 if [ -n "$1" ] && [ -e "$1" ]; then
  local dir="$(dirname $1)";
  local name="$(basename $1)";
  [ -e $dir/backup ] || mkdir $dir/backup;
  mv -f $1 $dir/backup/$name && { echo "cleanup: $1 => $dir/backup/$name"; };
 fi;
};

function diesel_open(){
 local filename="$initscript";
 if [ -e $idehistory ]; then
  local last="$(cat $idehistory | tail -n 1)";
  [ -n "$last" ] && [ -e "$last" ] && filename="$last";
 fi;
 echo "last open: $filename";
 local script="$(\
        zenity --title 'Open Diesel Pascal Project …'   \
        --window-icon $icon_ide                         \
        --file-selection --modal                        \
        --filename $filename                            \
        --file-filter 'LM9|*.[Ll][Mm]9'                 \
        --file-filter 'ALL|*.*'                         \
        2>/dev/null)";
 if [ -n "$script" ] && [ -e "$script" ]; then
  echo "this open: $script";
  saveuniq $idehistory $script;
  launch /opt/diesel/CrossDesigner $script;
 fi;
};

function diesel_run(){
 local filename="$initscript";
 if [ -e $runhistory ]; then
  local last="$(cat $runhistory | tail -n 1)";
  [ -n "$last" ] && [ -e "$last" ] && filename="$last";
 fi;
 echo "last run: $filename";
 local script="$(\
        zenity --title 'Run Diesel Pascal Project …'    \
        --window-icon $icon_run                         \
        --file-selection --modal                        \
        --filename $filename                            \
        --file-filter 'LM9|*.[Ll][Mm]9'                 \
        --file-filter 'ALL|*.*'                         \
        2>/dev/null)";
 if [ -n "$script" ] && [ -e "$script" ]; then
  echo "this run: $script";
  saveuniq $runhistory $script;
  launch /opt/diesel/CrossMachine $script;
 fi;
};

function diesel_defaults(){
 backup_file $envoptconf;
 backup_file $syncolconf;
 /opt/diesel/settings/diesel-defaults.sh;
};

function diesel_cleanup(){
 cleanup_backup_file $runhistory;
 cleanup_backup_file $idehistory;
 cleanup_backup_file $txthistory;
};

function diesel_cpl_exec(){
 case "$1" in
  Open|open)
   diesel_open;
   ;;
  IDE|ide|New|new|CrossDesigner)
   launch /opt/diesel/CrossDesigner;
   ;;
  Run|run|CrossMachine)
   diesel_run;
   ;;
  Manager|manager|DManager)
   launch /opt/diesel/DManager;
   ;;
  Defaults|defaults|diesel-defaults.sh)
   diesel_defaults;
   ;;
  Cleanup|cleanup)
   diesel_cleanup;
   ;;
  *)
   echo "${0##*/} {Open|IDE|New|CrossDesigner|Run|CrossMachine|Manager|DManager|Cleanup|Defaults|diesel-defaults.sh}"
 esac;
};

function diesel_cpl_menu(){
 local menu="$(\
    zenity --title 'Diesel Pascal Control Panel …' --width 500 --height 250     \
    --text 'Diesel Pascal: Выбирете желаемое действие:' --timeout 180 --modal   \
    --window-icon $icon_ide                                                     \
    --list --column ID --column Действие --column Action --print-column 1       \
    Run         'Запуск проекта    '    'Run  Diesel Pascal Project'            \
    Open        'Открыть проект    '    'Open Diesel Pascal Project'            \
    IDE         'Открыть дизайнер  '    'Diesel Pascal Designer IDE'            \
    Manager     'Менеджер проектов '    'Diesel Pascal Project Manager'         \
    Cleanup     'Очистка истории   '    'Clear history of run projects'         \
    Defaults    'Сброс настроек    '    'Reset IDE Setting to Defaults'         \
    2>/dev/null)";
 [ -n "$menu" ] && diesel_cpl_exec "$menu";
};

function main(){
 about; sleep 1;
 diesel_cpl_menu "$@";
};

main "$@";

#############
# END OF FILE
#############
